/*
 * Copyright (c) 2006,2008 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 ** alog client program
 **
 **/
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include <netdb.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

void
usage(char *progname)
{
	fprintf(stderr,
	    "usage: %s [-b][-n name][-h host][-p port][--] command [arg] ...\n",
	    progname);
	exit(2);
}

int
main(int argc, char *argv[])
{
	int ch, fd, n, opt_bg = 0;
	pid_t pid;
	char *progname = argv[0];
	char *name = NULL, *server = NULL, *port = NULL, *cause;
	struct addrinfo hints, *addri, *addr;
	
	/* Parse options */
	n = 0;
	while ((ch = getopt(argc, argv, "bn:h:p:")) != -1) {
		switch (ch) {
		case 'b':
			opt_bg++;
			break;
		case 'n':
			name = optarg;
			break;
		case 'h':
			server = optarg;
			break;
		case 'p':
			port = optarg;
			break;
		default:
			usage(progname);
		}
	}
	argc -= optind;
	argv += optind;
	if (argc == 0) usage(progname);
	
	if (server == NULL)
		server = getenv("ALOG_HOST");
	if (server == NULL)
		server = "localhost";
	if (port == NULL) port = "3412";
	if (name == NULL) name = argv[0];
	
	/* Connect to alog server */
	memset(&hints, 0, sizeof(hints));
	hints.ai_family = PF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_protocol = IPPROTO_TCP;
	hints.ai_flags = 0;
	if (getaddrinfo(server, port, &hints, &addri) == -1) {
		perror("getaddrinfo");
		exit(2);
	}
	for (addr = addri; addr != NULL; addr = addr->ai_next) {
		fd = socket(addr->ai_family, addr->ai_socktype,
		    addr->ai_protocol);
		if (fd < 0) {
			cause = "socket";
			continue;
		}
		if (connect(fd, addr->ai_addr, addr->ai_addrlen) == -1) {
			cause = "connect";
			close(fd);
			fd = -1;
			continue;
		}
		break;		/* good */
	}
	freeaddrinfo(addri);
	if (fd == -1) {
		fprintf(stderr, "could not connect to %s: %s: %s\n", server,
		    cause, strerror(errno));
		exit(2);
	}
	if (opt_bg) {
		/* fork */
		pid = fork();
		if (pid < 0) {
			perror("fork");
			exit(2);
		}
		if (pid != 0) {
			/* father */
			exit(0);
		}
	}
	/* Send client id */
	write(fd, name, strlen(name));
	write(fd, "\n", 1);
	
	/* redirect outputs */
	dup2(fd, fileno(stdout));
	dup2(fd, fileno(stderr));
	
	/* Execute program */
	if (execvp(argv[0], argv) == -1) {
		perror("execv");
		exit(2);
	}
	return 0;
}
